from math import floor

from features.moving_feature import moving_feature
from features.standard_deviation import standard_deviation
from korpus import create_bewegung

CLASS_JOGGEN = 1
CLASS_KNIEBEUGE = -1

TRAINING_DATA_PERCENTAGE = 0.9

DATA_LIMIT = 1000
DATA_LIMIT_PER_TYPE = floor(DATA_LIMIT / 2)


def binary_classification_feature(window_size=30):
    # Wir holen uns den Höhensensor des Fußes (Bleibt bei Kniebeugen gleich und bewegt sich beim Laufen)
    joggen_values = create_bewegung('joggen').messungen[1].sensoren[1].werte
    kniebeuge_values = create_bewegung('kniebeuge').messungen[1].sensoren[1].werte

    # Berechne die Standardabweichung (Bei Kniebeugen gering, bei Joggen hoch)
    joggen_feature = moving_feature(standard_deviation, window_size, joggen_values)
    kniebeuge_feature = moving_feature(standard_deviation, window_size, kniebeuge_values)

    return joggen_feature, kniebeuge_feature


def get_labeled_testdata():
    # Hole die aus den Sensordaten berechneten Merkmale
    joggen_feature, kniebeuge_feature = binary_classification_feature(150)

    # Wir nehmen nur DATA_LIMIT an Daten (sonst ist K-Nearest-Neighbors zu langsam)
    joggen_feature = joggen_feature[: min(DATA_LIMIT_PER_TYPE, len(joggen_feature))]
    kniebeuge_feature = kniebeuge_feature[: min(DATA_LIMIT_PER_TYPE, len(kniebeuge_feature))]

    # Wandel Liste an Merkmalen in einzelne Merkmalsvektoren um
    joggen_vector = ([element] for element in joggen_feature)
    kniebeugen_vector = ([element] for element in kniebeuge_feature)

    # Weise den Trainingsdaten eine Klasse zu
    # 0 = Kniebeuge, 1 = Joggen
    training_data_joggen = list(zip(joggen_vector, [CLASS_JOGGEN] * len(joggen_feature)))
    training_data_kniebeuge = list(zip(kniebeugen_vector, [CLASS_KNIEBEUGE] * len(kniebeuge_feature)))

    # Wir nehmen 90 % der Testdaten zum Trainieren und 10 % zum Testen
    delimiter_joggen = floor(len(joggen_feature) * TRAINING_DATA_PERCENTAGE)
    delimiter_kniebeuge = floor(len(kniebeuge_feature) * TRAINING_DATA_PERCENTAGE)

    training_data = training_data_joggen[:delimiter_joggen] + training_data_kniebeuge[:delimiter_kniebeuge]
    test_data = training_data_joggen[delimiter_joggen:] + training_data_kniebeuge[delimiter_kniebeuge:]

    return test_data, training_data
